package digital.laboratory.platform.entrustment.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.entrustment.entity.EntrustTemplateInfo;
import digital.laboratory.platform.entrustment.enums.TemplateTypeEnums;
import digital.laboratory.platform.entrustment.query.EntrustTemplateInfoQuery;
import digital.laboratory.platform.entrustment.service.EntrustTemplateInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 委托检材--检出定性定量结果信息
 *
 * @author chenjiangbao
 * @describe 委托检材--检出定性定量结果信息相关接口 前端控制器
 */
@RestController
@RequestMapping("/papp/entrustTemplateInfo")
@Api(tags = "019-委托送检中案情简要等内容的模板信息相关接口")
public class EntrustTemplateInfoController {

    @Resource
    private EntrustTemplateInfoService entrustTemplateInfoService;

    // 内部静态类，用于封装枚举数据
    private static class EnumResponse {
        private final int code;
        private final String desc;

        public EnumResponse(int code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        public int getCode() {
            return code;
        }

        public String getDesc() {
            return desc;
        }
    }

    @ApiOperation("获取所有模板的类型值")
    @GetMapping("/allTemplateType")
    public R<List<EnumResponse>> allTemplateType() {
        List<EnumResponse> enumResponseList = Arrays.stream(TemplateTypeEnums.values())
                .map(value -> new EnumResponse(value.getCode(), value.getDesc()))
                .collect(Collectors.toList());
        return R.ok(enumResponseList);
    }

    @ApiOperation("获取模板的分页接口")
    @PostMapping("/page")
    public R<IPage<EntrustTemplateInfo>> pageEntrustTemplateInfo(@RequestBody EntrustTemplateInfoQuery query) {
        return R.ok(
                entrustTemplateInfoService.page(
                        new Page<>(query.getCurrent(), query.getSize()),
                        Wrappers.<EntrustTemplateInfo>lambdaQuery()
                                .eq(query.getType() != null, EntrustTemplateInfo::getTemplateType, query.getType())
                                .orderByDesc(EntrustTemplateInfo::getCreateTime)
                )
        );
    }

    @ApiOperation("录入模板信息")
    @PostMapping("/save")
    public R<EntrustTemplateInfo> addEntrustTemplateInfo(@RequestBody @Valid EntrustTemplateInfo info) {
        EntrustTemplateInfo isExist = entrustTemplateInfoService.getOne(
                Wrappers.<EntrustTemplateInfo>lambdaQuery()
                        .eq(EntrustTemplateInfo::getTemplateName, info.getTemplateName())
                        .eq(EntrustTemplateInfo::getTemplateType, info.getTemplateType())
        );
        if (isExist != null) {
            return R.failed(String.format("在模板类型为 [%s] 中已经存在模板名称为 [%s] 的模板！",
                    TemplateTypeEnums.fromCode(info.getTemplateType()).getDesc(),
                    info.getTemplateName()));
        }
        entrustTemplateInfoService.save(info);
        return R.ok(info);
    }

    @ApiOperation("更新模板信息")
    @PutMapping("/update")
    public R<EntrustTemplateInfo> update(@RequestBody EntrustTemplateInfo info) {
        EntrustTemplateInfo isExist = entrustTemplateInfoService.getById(info.getId());
        if (isExist == null) {
            return R.failed("数据在系统中不存在！");
        }
        entrustTemplateInfoService.updateById(info);
        return R.ok(info);
    }

    @ApiOperation("删除模板信息")
    @PostMapping("/delete")
    public R<Boolean> delete(@RequestBody List<String> ids) {
        return R.ok(entrustTemplateInfoService.removeByIds(ids));
    }
}
